#version 130

in vec2 coord;
out vec4 fragment;
uniform sampler2D aoMap;
uniform sampler2D depthMap;

uniform vec4 resolution;	// Input texture resolution
uniform float falloff;		// Blur falloff factor
uniform float sharpness;	// Sharpness with depth adjustment
uniform float radius;		// Blur radius

void main() {
	vec2 halfRes = 2.0 * resolution.zw;
	float blur = texture2D(aoMap, coord).r;
	float depth = texture2D(depthMap, coord).r;
	float total = 1.0;

	for(float r = -radius; r<=radius; ++r) {
		vec2 sample = coord + vec2(r*halfRes, 0).xy; // use .yx for y axis blur. use preprocessor
		float c = texture2D(aoMap, sample).r;
		float d = texture2D(depthMap, sample).r;
		float w = exp(-r*r*falloff - d*d*sharpness);
		total += w;
		blur += w * c;
	}

	fragment = vec4(blur/total);
}





