#version 130

#pragma vertex_shader

in vec4 vertex;
in vec3 normal;
in vec4 tangent;
in vec2 texCoord;
in float weights; // Type index


uniform mat4  transform;
uniform mat4  modelMatrix;

out vec2 texcoord;
out vec3 worldNormal;
out vec3 worldPos;
out vec3 worldTangent;
out vec3 worldBinormal;
out vec4 colour;

void main() {
	vec4 pos = vertex;
	vec3 norm = normal;
	vec3 tang = tangent.xyz;
	
	gl_Position = transform * pos;
	texcoord = texCoord;
	worldNormal = mat3(modelMatrix) * norm;
	worldTangent = mat3(modelMatrix) * tang;
	worldBinormal = cross(worldNormal, worldTangent) * (tangent.w * 2 - 1);
	worldPos = (modelMatrix * pos).xyz;

	if(worldNormal.z > 0.90) {
		texcoord = worldPos.xy * 0.137;
		worldTangent = vec3(0,1,0);
		worldBinormal = vec3(-1,0,0);
		if(worldPos.z < -1) texcoord += 0.5;
	}

	vec4 colours[4];
	colours[0] = vec4(0.5,0.3,0.1,1);
	colours[1] = vec4(0.9,0.65,0.1,0.1);
	colours[2] = vec4(0.6,0.6,0.6,1);
	colours[3] = vec4(0.2,0.1,0.2,1);
	colour = colours[int(weights)];
}


#pragma fragment_shader

in vec2 texcoord;
in vec3 worldNormal;
in vec3 worldTangent;
in vec3 worldBinormal;
in vec3 worldPos;
in vec4 colour;
out vec4 buf0;
out vec4 buf1;

uniform sampler2D diffusemap;
uniform sampler2D normalmap;

void main() {

	// Full Triplanar
	vec2 tc = worldPos.xy * 0.137;
	if(abs(worldNormal.x) > 0.3) tc = worldPos.yz * vec2(0.321, 0.5);
	if(abs(worldNormal.y) > 0.3) tc = worldPos.xz * vec2(0.321, 0.5); 
	if(worldNormal.z>0.3) tc = worldPos.xy * 0.137;
	float flip = worldNormal.x<-0.3 || worldNormal.y > 0.3? -1: 1;
	

	vec4 diff = texture2D(diffusemap, texcoord.st);
	vec4 norm = texture2D(normalmap, tc.st); 


	diff *= colour;

	if(worldPos.z<-2.99) diff.rgb *= 0.8;

	// Normal map
	vec3 n = norm.xyz * 2.0 - 1.0;
	mat3 tbn = mat3(normalize(worldTangent*flip), normalize(worldBinormal*flip), normalize(worldNormal));
	vec3 normal = tbn * n;

	buf0 = diff;
	buf1 = vec4(normal * 0.5 + 0.5, 0);
}

